# -*- coding: utf-8 -*-
import sphinx_rtd_theme
import bandoleers

project = 'bandoleers'
copyright = 'AWeber Communications, Inc.'
version = bandoleers.__version__
release = '.'.join(str(v) for v in bandoleers.version_info[0:2])

needs_sphinx = '1.0'
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx',
    'sphinx.ext.viewcode',
]

templates_path = []
source_suffix = '.rst'
source_encoding = 'utf-8-sig'
master_doc = 'index'
exclude_patterns = []
pygments_style = 'sphinx'
html_theme = 'sphinx_rtd_theme'
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
html_static_path = []

intersphinx_mapping = {
    'python': ('http://docs.python.org/3/', None),
    'rejected': ('http://rejected.readthedocs.org/en/latest/', None),
    'consulate': ('http://consulate.readthedocs.org/en/latest/', None),
    'datastax': ('http://datastax.github.io/python-driver', None),
    'requests': ('http://requests.readthedocs.io/en/master/', None),
}



"""
Platform service configuration utility.

The prepit utility looks for a set of configuratione files in a local
directory called platform, and attempts to apply those configuration
settings to platform services that are discovered via environment
variables. See the README file in the git repo for this utility
for more details.

"""
import logging
import os.path
import re
import sys
try:
    from urllib.parse import urljoin, urlsplit, urlunsplit
except ImportError:
    from urlparse import urljoin, urlsplit, urlunsplit

from redis import StrictRedis
import json
import psycopg
import requests

from bandoleers import args


LOGGER = logging.getLogger(__name__)


def prep_redis(file_):
    try:
        LOGGER.info('Processing %s', file_)
        parts = urlsplit(os.environ.get('REDIS_URI', 'redis://localhost'))
        redis = StrictRedis(host=parts.hostname,
                            port=parts.port or 6379,
                            db=parts.path[1:] or 0)

        with open(file_) as fh:
            config = json.load(fh)
            for command, entries in config.items():
                for name, values in entries.items():
                    redis.execute_command(command, name, *values)
        redis.connection_pool.disconnect()
    except Exception:
        LOGGER.exception('Failed to execute redis commands.')
        sys.exit(-1)


def prep_consul(file):
    try:
        LOGGER.info('Processing %s', file)
        kv_root = urlunsplit((
            'http',
            '{}:{}'.format(os.environ.get('CONSUL_HOST', 'localhost'),
                           os.environ.get('CONSUL_PORT', '8500')),
            '/v1/kv/', None, None))

        with open(file) as fh:
            config = json.load(fh)
            LOGGER.debug('%r', config)
            with requests.Session() as session:
                for k, v in config.items():
                    rsp = session.put(urljoin(kv_root, k.lstrip('/')),
                                      data=str(v).encode())
                    rsp.raise_for_status()
    except Exception:
        LOGGER.exception('Failed to load consul data.')
        sys.exit(-1)


def prep_postgres(file):
    try:
        LOGGER.info('Processing %s', file)
        db = os.path.splitext(os.path.basename(file))[0]
        uri = os.environ.get('PGSQL_{0}'.format(db.upper()))
        if uri is not None:
            chop = len(db) + 1
            base = uri[:-chop]
        else:
            base = os.environ.get('PGSQL',
                                  'postgresql://postgres@localhost:5432')
            uri = os.path.join(base, db)
            conn = psycopg.connect(os.path.join(base, 'postgres'),
                                   autocommit=True)
            with conn.cursor() as cursor:
                LOGGER.debug('Creating database')
                cursor.execute('DROP DATABASE IF EXISTS {}'.format(db))
                cursor.execute('CREATE DATABASE {}'.format(db))

        with open(file) as fh:
            sql = fh.read()
        conn = psycopg.connect(uri, autocommit=True)
        with conn.cursor() as cursor:
            cursor.execute(sql)
    except Exception:
        LOGGER.exception('Failed to execute pgsql queries.')
        sys.exit(-1)


def prep_rabbit(file):
    try:
        LOGGER.info('Processing %s', file)
        host = os.environ.get('RABBITMQ', 'localhost')
        with open(file) as fh:
            config = json.load(fh)
            with requests.Session() as session:
                for action in config:
                    uri = 'http://{0}/{1}'.format(host, action['path'])
                    LOGGER.debug('%s', action)
                    r = session.request(url=uri, method=action['method'],
                                        auth=('guest', 'guest'),
                                        json=action['body'])
                    r.raise_for_status()
    except Exception:
        LOGGER.exception('Failed to configure rabbit.')
        sys.exit(-1)


def prep_http(file):
    LOGGER.info('Processing %s', file)
    var_pattern = re.compile(r'\$(?P<name>[_a-z0-9]+)',
                             re.IGNORECASE)
    with open(file) as fh:
        input_requests = json.load(fh)
        with requests.Session() as session:
            for request in input_requests:
                try:
                    url = request['url']
                    matches = var_pattern.findall(url)
                    for var_name in matches:
                        if var_name in os.environ:
                            url = url.replace('${}'.format(var_name),
                                              os.environ[var_name])
                    parts = urlsplit(url)
                    user, password = parts.username, parts.password
                    hostname, port = parts.hostname, parts.port
                    if port:
                        netloc = '{}:{}'.format(hostname, port)
                    else:
                        netloc = hostname

                    request['url'] = urlunsplit((
                        parts.scheme, netloc, parts.path, parts.query,
                        parts.fragment))
                    request.setdefault('method', 'GET')
                    request.setdefault('auth', (user, password))
                    LOGGER.debug('making HTTP request %r', request)
                    r = session.request(**request)
                    r.raise_for_status()
                except Exception:
                    LOGGER.exception('HTTP request %r failed.', request)
                    sys.exit(-1)


def run():
    logging.basicConfig(
        level=logging.INFO,
        format='%(levelname)1.1s - %(name)s: %(message)s')
    parser = args.ArgumentParser()
    parser.add_argument('-d', '--dir', metavar='DIRECTORY',
                        default='platform', dest='directory',
                        help='read files from DIRECTORY')
    opts = parser.parse_args()

    resources = {
        'consul': prep_consul,
        'http': prep_http,
        'postgres': prep_postgres,
        'rabbitmq': prep_rabbit,
        'redis': prep_redis,
    }

    top_level = os.listdir(opts.directory)
    for resource in resources:
        if resource in top_level:
            path = '/'.join([opts.directory, resource])
            files = os.listdir(path)
            for data_file in files:
                if data_file.startswith('.') or data_file.endswith('~'):
                    continue
                resources[resource]('/'.join([path, data_file]))
    LOGGER.info('Finished processing successfully.')
    sys.exit(0)


if __name__ == '__main__':
    run()



"""
Wait for a URL to become available.

"""
import logging
import socket
import sys
import time
import warnings
try:
    from urllib import parse
except ImportError:
    import urlparse as parse

warnings.simplefilter('ignore', UserWarning)

import psycopg
import requests.exceptions

from bandoleers import args


LOGGER = logging.getLogger(__name__)


def connect_to(url, timeout):
    scheme, netloc, path, query, fragment = parse.urlsplit(url)
    if scheme in ('http', 'https'):
        response = requests.get(url, timeout=timeout)
        response.raise_for_status()

    elif scheme == 'postgresql':
        kwargs = {
            'user': 'postgres',
            'password': None,
            'host': 'localhost',
            'port': 5432,
            'dbname': 'postgres',
        }
        user_n_pass, sep, host_n_port = netloc.partition('@')
        if sep:
            user, sep, password = user_n_pass.partition(':')
            kwargs['user'] = user
            if sep:
                kwargs['password'] = password
        else:
            host_n_port = user_n_pass

        host, sep, port = host_n_port.partition(':')
        kwargs['host'] = host
        if sep:
            kwargs['port'] = int(port)

        if path:
            kwargs['dbname'] = path[1:]

        LOGGER.debug('connecting to postgres with %r', kwargs)
        conn = psycopg.connect(**kwargs)
        conn.close()

    elif scheme == 'tcp':
        host, sep, port = netloc.partition(':')
        if not sep:
            raise RuntimeError('tcp:// requires a port')
        try:
            port = int(port)
        except:
            raise RuntimeError('failed to extract port from ' + netloc)

        ip_addr = socket.gethostbyname(host)
        LOGGER.debug('opening TCP connection to %r:%r', ip_addr, port)
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM,
                             socket.IPPROTO_TCP)
        sock.settimeout(timeout)
        sock.connect((ip_addr, port))
        sock.close()

    else:
        raise RuntimeError("I don't know what to do with {0}".format(scheme))


def run():
    logger = logging.getLogger(__name__)
    logging.basicConfig(
        level=logging.INFO,
        format='%(relativeCreated)-10.3f %(levelname)-8s %(message)s')

    parser = args.ArgumentParser()
    parser.add_argument('-s', '--sleep', type=float, metavar='SECONDS',
                        default=0.25, help='time to sleep after a failure')
    parser.add_argument('-t', '--timeout', type=float, metavar='SECONDS',
                        default=None,
                        help=('total number of seconds to run for. '
                              'Default is to run forever'))
    parser.add_argument('URL')
    opts = parser.parse_args()

    t0 = time.time()
    wait_forever = opts.timeout is None
    timeout = max(opts.sleep, opts.timeout or opts.sleep)
    logger.debug('waiting on %s for %s', opts.URL,
                 'forever' if wait_forever else opts.timeout)

    while True:
        try:
            connect_to(opts.URL, timeout=timeout)
            logger.debug('connection to %s succeeded after %f seconds',
                         opts.URL, time.time() - t0)
            sys.exit(0)

        except RuntimeError:
            logger.exception('internal failure')
            sys.exit(70)

        except KeyboardInterrupt:
            logger.info('killed')
            sys.exit(-1)

        except Exception as error:
            if not wait_forever and (time.time() - t0) >= opts.timeout:
                break

            logger.debug('%r, sleeping for %f seconds', error, opts.sleep)
            time.sleep(opts.sleep)

    logger.error('wait timed out after %f seconds', time.time() - t0)
    sys.exit(-1)


if __name__ == '__main__':
    run()



import argparse
import logging
import sys

import bandoleers


class ArgumentParser(argparse.ArgumentParser):
    """
    Implements some common command-line behaviors.

    This is a slightly extended version of the standard
    :class:`argparse.ArgumentParser` class that does three
    things for you:

    * exits with a non-zero status when help is shown
    * implements --quiet and --verbose mutually exclusive
      options that call :meth:`logging.Logger.setLevel` on
      the root logger
    * adds a --version flag

    """

    def __init__(self, *args, **kwargs):
        super(ArgumentParser, self).__init__(*args, **kwargs)
        output_control = self.add_mutually_exclusive_group()
        output_control.add_argument('-q', '--quiet',
                                    action='store_true', default=False,
                                    help='disable non-failure output')
        output_control.add_argument('-v', '--verbose',
                                    action='store_true', default=False,
                                    help='show diagnostic output')
        self.add_argument(
            '--version', action='version',
            version='%(prog)s {0}'.format(bandoleers.__version__))

    def parse_args(self, *args, **kwargs):
        result = super(ArgumentParser, self).parse_args(*args, **kwargs)
        if result.verbose:
            logging.getLogger().setLevel(logging.DEBUG)
        if result.quiet:
            logging.getLogger().setLevel(logging.ERROR)
        return result

    def print_usage(self, file=None):
        stream = file or sys.stdout
        stream.write(self.format_usage())
        sys.exit(64)

    def print_help(self, file=None):
        stream = file or sys.stdout
        stream.write(self.format_help())
        sys.exit(64)



#!/usr/bin/env python
#
import os.path

import setuptools

import bandoleers


def read_requirements(name):
    requirements = []
    try:
        with open(os.path.join('requires', name)) as req_file:
            for line in req_file:
                if '#' in line:
                    line = line[:line.index('#')]
                line = line.strip()
                if line.startswith('-r'):
                    requirements.extend(read_requirements(line[2:].strip()))
                elif line and not line.startswith('-'):
                    requirements.append(line)
    except IOError:
        pass
    return requirements


setuptools.setup(
    name='bandoleers',
    description='AWeber development tool belt',
    long_description='\n'+open('README.rst').read(),
    version=bandoleers.__version__,
    packages=setuptools.find_packages(exclude=['tests', 'tests.*']),
    license='BSD',
    url='https://github.com/aweber/bandoleers',
    author='AWeber Communications, Inc.',
    author_email='api@aweber.com',
    install_requires=read_requirements('installation.txt'),
    tests_require=read_requirements('testing.txt'),
    entry_points={
        'console_scripts': [
            'prep-it=bandoleers.prepit:run',
            'wait-for=bandoleers.waitfor:run',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)
